package de.fhdw.wtf.dsl.scanner.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.CommentTextToken;
import de.fhdw.wtf.common.token.EndToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.InvalidToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.WhitespaceToken;
import de.fhdw.wtf.common.token.keywords.AbstractToken;
import de.fhdw.wtf.common.token.keywords.ClassToken;
import de.fhdw.wtf.common.token.keywords.ExceptionToken;
import de.fhdw.wtf.common.token.keywords.FindableToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.keywords.MutableToken;
import de.fhdw.wtf.common.token.keywords.PriorToken;
import de.fhdw.wtf.common.token.keywords.ServiceToken;
import de.fhdw.wtf.common.token.keywords.SymmetricToken;
import de.fhdw.wtf.common.token.keywords.TransientToken;
import de.fhdw.wtf.common.token.symbols.ArrowToken;
import de.fhdw.wtf.common.token.symbols.AsteriskToken;
import de.fhdw.wtf.common.token.symbols.BracketCloseToken;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.ColonToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.DoubleSquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.DoubleSquareBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.EqualToken;
import de.fhdw.wtf.common.token.symbols.ExclamationToken;
import de.fhdw.wtf.common.token.symbols.GreaterSymbolToken;
import de.fhdw.wtf.common.token.symbols.PlusSymbolToken;
import de.fhdw.wtf.common.token.symbols.SemicolonToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;

/**
 * Test cases for operations provided by the Scanner.
 * 
 */
public class ScannerTest {
	
	@Test
	public void digitIdentifierTest() {
		final SimpleScannerInput input = new SimpleScannerInput("9Hallo");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(InvalidToken.create("9", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(IdentifierToken.create("Hallo", Position.create("", 0 + 1, 1 + 1, 1)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 6 + 1, 6)), output.next());
	}
	
	@Test
	public void singleSymol() {
		final SimpleScannerInput input = new SimpleScannerInput("[]{}+:;=>*");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(true, output.hasNext());
		assertEquals(SquareBracketOpenToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(SquareBracketCloseToken.create(Position.create("", 0 + 1, 1 + 1, 1)), output.next());
		assertEquals(CurlyBracketOpenToken.create(Position.create("", 0 + 1, 2 + 1, 2)), output.next());
		assertEquals(CurlyBracketCloseToken.create(Position.create("", 0 + 1, 3 + 1, 3)), output.next());
		assertEquals(PlusSymbolToken.create(Position.create("", 0 + 1, 4 + 1, 4)), output.next());
		assertEquals(ColonToken.create(Position.create("", 0 + 1, 5 + 1, 5)), output.next());
		assertEquals(SemicolonToken.create(Position.create("", 0 + 1, 6 + 1, 6)), output.next());
		assertEquals(EqualToken.create(Position.create("", 0 + 1, 7 + 1, 7)), output.next());
		assertEquals(GreaterSymbolToken.create(Position.create("", 0 + 1, 8 + 1, 8)), output.next());
		assertEquals(AsteriskToken.create(Position.create("", 0 + 1, 9 + 1, 9)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 10 + 1, 10)), output.peek());
		output.removeFirst();
		assertEquals(false, output.hasNext());
	}
	
	@Test
	public void keywords() {
		final SimpleScannerInput input = new SimpleScannerInput("abstract class group prior");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(AbstractToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 8 + 1, 8)), output.next());
		assertEquals(ClassToken.create(Position.create("", 0 + 1, 9 + 1, 9)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 14 + 1, 14)), output.next());
		assertEquals(GroupToken.create(Position.create("", 0 + 1, 15 + 1, 15)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 20 + 1, 20)), output.next());
		assertEquals(PriorToken.create(Position.create("", 0 + 1, 21 + 1, 21)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 26 + 1, 26)), output.next());
	}
	
	@Test
	public void identifiers() {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Bond007 what_is_the_answer_to_life_the_universe_and_everything  ThreadsafeButton_Channel SimpleNodeTester");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("Bond007", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 7 + 1, 7)), output.next());
		assertEquals(
				IdentifierToken.create(
						"what_is_the_answer_to_life_the_universe_and_everything",
						Position.create("", 0 + 1, 8 + 1, 8)),
				output.next());
		assertEquals(WhitespaceToken.create("  ", Position.create("", 0 + 1, 62 + 1, 62)), output.next());
		assertEquals(
				IdentifierToken.create("ThreadsafeButton_Channel", Position.create("", 0 + 1, 64 + 1, 64)),
				output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 88 + 1, 88)), output.next());
		assertEquals(IdentifierToken.create("SimpleNodeTester", Position.create("", 0 + 1, 89 + 1, 89)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 105 + 1, 105)), output.next());
	}
	
	@Test
	public void classInGroup() {
		final SimpleScannerInput input = new SimpleScannerInput("ichbingruppe:group=[IchBinEINEkLASSE:class={}]");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("ichbingruppe", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(ColonToken.create(Position.create("", 0 + 1, 12 + 1, 12)), output.next());
		assertEquals(GroupToken.create(Position.create("", 0 + 1, 13 + 1, 13)), output.next());
		assertEquals(EqualToken.create(Position.create("", 0 + 1, 18 + 1, 18)), output.next());
		assertEquals(SquareBracketOpenToken.create(Position.create("", 0 + 1, 19 + 1, 19)), output.next());
		assertEquals(IdentifierToken.create("IchBinEINEkLASSE", Position.create("", 0 + 1, 20 + 1, 20)), output.next());
		assertEquals(ColonToken.create(Position.create("", 0 + 1, 36 + 1, 36)), output.next());
		assertEquals(ClassToken.create(Position.create("", 0 + 1, 37 + 1, 37)), output.next());
		assertEquals(EqualToken.create(Position.create("", 0 + 1, 42 + 1, 42)), output.next());
		assertEquals(CurlyBracketOpenToken.create(Position.create("", 0 + 1, 43 + 1, 43)), output.next());
		assertEquals(CurlyBracketCloseToken.create(Position.create("", 0 + 1, 44 + 1, 44)), output.next());
		assertEquals(SquareBracketCloseToken.create(Position.create("", 0 + 1, 45 + 1, 45)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 46 + 1, 46)), output.next());
	}
	
	@Test
	public void testInvalidChar() {
		final SimpleScannerInput input = new SimpleScannerInput("^ichbingruppe:group=[IchBinEINEkLASSE:class^={}]");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(InvalidToken.create('^', Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(IdentifierToken.create("ichbingruppe", Position.create("", 0 + 1, 1 + 1, 1)), output.next());
		assertEquals(ColonToken.create(Position.create("", 0 + 1, 13 + 1, 13)), output.next());
		assertEquals(GroupToken.create(Position.create("", 0 + 1, 14 + 1, 14)), output.next());
		assertEquals(EqualToken.create(Position.create("", 0 + 1, 19 + 1, 19)), output.next());
		assertEquals(SquareBracketOpenToken.create(Position.create("", 0 + 1, 20 + 1, 20)), output.next());
		assertEquals(IdentifierToken.create("IchBinEINEkLASSE", Position.create("", 0 + 1, 21 + 1, 21)), output.next());
		assertEquals(ColonToken.create(Position.create("", 0 + 1, 37 + 1, 37)), output.next());
		assertEquals(ClassToken.create(Position.create("", 0 + 1, 38 + 1, 38)), output.next());
		assertEquals(InvalidToken.create('^', Position.create("", 0 + 1, 43 + 1, 43)), output.next());
		assertEquals(EqualToken.create(Position.create("", 0 + 1, 44 + 1, 44)), output.next());
		assertEquals(CurlyBracketOpenToken.create(Position.create("", 0 + 1, 45 + 1, 45)), output.next());
		assertEquals(CurlyBracketCloseToken.create(Position.create("", 0 + 1, 46 + 1, 46)), output.next());
		assertEquals(SquareBracketCloseToken.create(Position.create("", 0 + 1, 47 + 1, 47)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 48 + 1, 48)), output.next());
	}
	
	@Test
	public void testClassNames() {
		final SimpleScannerInput input = new SimpleScannerInput("IchBinEINEkLASSE:class={};className:class={};");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("IchBinEINEkLASSE", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertTrue(output.next() instanceof ColonToken);
		assertTrue(output.next() instanceof ClassToken);
		assertTrue(output.next() instanceof EqualToken);
		assertTrue(output.next() instanceof CurlyBracketOpenToken);
		assertTrue(output.next() instanceof CurlyBracketCloseToken);
		assertTrue(output.next() instanceof SemicolonToken);
		assertEquals(IdentifierToken.create("className", Position.create("", 0 + 1, 26 + 1, 26)), output.next());
		assertTrue(output.next() instanceof ColonToken);
		assertTrue(output.next() instanceof ClassToken);
		assertTrue(output.next() instanceof EqualToken);
		assertTrue(output.next() instanceof CurlyBracketOpenToken);
		assertTrue(output.next() instanceof CurlyBracketCloseToken);
		assertTrue(output.next() instanceof SemicolonToken);
		assertEquals(EndToken.create(Position.create("", 0 + 1, 45 + 1, 45)), output.next());
	}
	
	@Test
	public void invalid() {
		final SimpleScannerInput input = new SimpleScannerInput("abstract  &hallo");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(AbstractToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create("  ", Position.create("", 0 + 1, 8 + 1, 8)), output.next());
		assertEquals(InvalidToken.create("&", Position.create("", 0 + 1, 10 + 1, 10)), output.next());
		assertEquals(IdentifierToken.create("hallo", Position.create("", 0 + 1, 11 + 1, 11)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 16 + 1, 16)), output.next());
	}
	
	@Test
	public void testComment1() {
		final SimpleScannerInput input = new SimpleScannerInput("#Ich bin ein Kommentar#");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(
				CommentTextToken.create("#Ich bin ein Kommentar#", Position.create("", 0 + 1, 0 + 1, 0)),
				output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 23 + 1, 23)), output.next());
	}
	
	@Test
	public void testComment2() {
		final SimpleScannerInput input = new SimpleScannerInput("class#Ich bin ein Kommentar#;");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertTrue(output.next() instanceof ClassToken);
		assertEquals(
				CommentTextToken.create("#Ich bin ein Kommentar#", Position.create("", 0 + 1, 5 + 1, 5)),
				output.next());
		assertTrue(output.next() instanceof SemicolonToken);
		assertEquals(EndToken.create(Position.create("", 0 + 1, 29 + 1, 29)), output.next());
	}
	
	@Test
	public void testComment3() {
		final SimpleScannerInput input = new SimpleScannerInput("#Ich bin ein Kommentar;");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(
				CommentTextToken.create("#Ich bin ein Kommentar;#", Position.create("", 0 + 1, 0 + 1, 0)),
				output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 23 + 1, 23)), output.next());
		
	}
	
	@Test
	public void testComment4() {
		final SimpleScannerInput input = new SimpleScannerInput("#auto:class={};");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(CommentTextToken.create("#auto:class={};#", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 15 + 1, 15)), output.next());
		
	}
	
	@Test
	public void testComment5() {
		final SimpleScannerInput input = new SimpleScannerInput("#######");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(CommentTextToken.create("##", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(CommentTextToken.create("##", Position.create("", 0 + 1, 2 + 1, 2)), output.next());
		assertEquals(CommentTextToken.create("##", Position.create("", 0 + 1, 4 + 1, 4)), output.next());
		assertEquals(CommentTextToken.create("##", Position.create("", 0 + 1, 6 + 1, 6)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 7 + 1, 7)), output.next());
		
	}
	
	@Test
	public void testComment6() {
		final SimpleScannerInput input = new SimpleScannerInput("/*Kommentar;*/");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(InvalidToken.create("/", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(AsteriskToken.create(Position.create("", 0 + 1, 1 + 1, 1)), output.next());
		assertEquals(IdentifierToken.create("Kommentar", Position.create("", 0 + 1, 2 + 1, 2)), output.next());
		assertEquals(SemicolonToken.create(Position.create("", 0 + 1, 11 + 1, 11)), output.next());
		assertEquals(AsteriskToken.create(Position.create("", 0 + 1, 12 + 1, 12)), output.next());
		assertEquals(InvalidToken.create("/", Position.create("", 0 + 1, 13 + 1, 13)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 14 + 1, 14)), output.next());
		
	}
	
	@Test
	public void testCommentEmpty() {
		final SimpleScannerInput input = new SimpleScannerInput("##");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(CommentTextToken.create("##", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 2 + 1, 2)), output.next());
	}
	
	@Test
	public void testWhitespace() {
		final SimpleScannerInput input = new SimpleScannerInput("abstract hallo class");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(AbstractToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 8 + 1, 8)), output.next());
		assertEquals(IdentifierToken.create("hallo", Position.create("", 0 + 1, 9 + 1, 9)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 14 + 1, 14)), output.next());
		assertEquals(ClassToken.create(Position.create("", 0 + 1, 15 + 1, 15)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 20 + 1, 20)), output.next());
	}
	
	@Test
	public void testWhitespace2() {
		final SimpleScannerInput input = new SimpleScannerInput("  abstract    hallo     class");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(WhitespaceToken.create("  ", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(AbstractToken.create(Position.create("", 0 + 1, 2 + 1, 2)), output.next());
		assertEquals(WhitespaceToken.create("    ", Position.create("", 0 + 1, 10 + 1, 10)), output.next());
		assertEquals(IdentifierToken.create("hallo", Position.create("", 0 + 1, 14 + 1, 14)), output.next());
		assertEquals(WhitespaceToken.create("     ", Position.create("", 0 + 1, 19 + 1, 19)), output.next());
		assertEquals(ClassToken.create(Position.create("", 0 + 1, 24 + 1, 24)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 29 + 1, 29)), output.next());
	}
	
	@Test
	public void testTab() {
		final SimpleScannerInput input = new SimpleScannerInput(" abstract	hallo	class");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(AbstractToken.create(Position.create("", 0 + 1, 1 + 1, 1)), output.next());
		assertEquals(WhitespaceToken.create("	", Position.create("", 0 + 1, 9 + 1, 9)), output.next());
		assertEquals(IdentifierToken.create("hallo", Position.create("", 0 + 1, 10 + 1, 10)), output.next());
		assertEquals(WhitespaceToken.create("	", Position.create("", 0 + 1, 15 + 1, 15)), output.next());
		assertEquals(ClassToken.create(Position.create("", 0 + 1, 16 + 1, 16)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 21 + 1, 21)), output.next());
	}
	
	@Test
	public void testInvalidCharacterInKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("cla&%ss");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("cla", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(InvalidToken.create("&%", Position.create("", 0 + 1, 3 + 1, 3)), output.next());
		assertEquals(IdentifierToken.create("ss", Position.create("", 0 + 1, 5 + 1, 5)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 7 + 1, 7)), output.next());
	}
	
	@Test
	public void testExclamationCharacterInKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("cla!ss");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("cla", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertTrue(output.next() instanceof ExclamationToken);
		assertEquals(IdentifierToken.create("ss", Position.create("", 0 + 1, 4 + 1, 4)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 6 + 1, 6)), output.next());
	}
	
	@Test
	public void testCommentCharacterInKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("cla#this is a class#ss");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("cla", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(CommentTextToken.create("#this is a class#", Position.create("", 0 + 1, 3 + 1, 3)), output.next());
		assertEquals(IdentifierToken.create("ss", Position.create("", 0 + 1, 20 + 1, 20)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 22 + 1, 22)), output.next());
	}
	
	@Test
	public void testWhitespaceInKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("cla   ss");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("cla", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create("   ", Position.create("", 0 + 1, 3 + 1, 3)), output.next());
		assertEquals(IdentifierToken.create("ss", Position.create("", 0 + 1, 6 + 1, 6)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 8 + 1, 8)), output.next());
	}
	
	@Test
	public void testExceptionAsKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("exception");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(ExceptionToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 9 + 1, 9)), output.next());
	}
	
	@Test
	public void testServiceAsKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("service");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(ServiceToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 7 + 1, 7)), output.next());
	}
	
	@Test
	public void testTransientAsKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("transient");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(TransientToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 9 + 1, 9)), output.next());
	}
	
	@Test
	public void testFindableAsKeyword() {
		final SimpleScannerInput input = new SimpleScannerInput("findable");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(FindableToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 8 + 1, 8)), output.next());
	}
	
	@Test
	public void testArrowAsToken() {
		final SimpleScannerInput input = new SimpleScannerInput("->");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(ArrowToken.create(Position.create("", 1, 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 1, 3, 2)), output.next());
	}
	
	@Test
	public void testDoubleSquareBracketOpenAsToken() {
		final SimpleScannerInput input = new SimpleScannerInput("[[");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(DoubleSquareBracketOpenToken.create(Position.create("", 1, 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 2 + 1, 2)), output.next());
	}
	
	@Test
	public void testDoubleSquareBracketCloseAsToken() {
		final SimpleScannerInput input = new SimpleScannerInput("]]");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(DoubleSquareBracketCloseToken.create(Position.create("", 1, 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 1, 3, 2)), output.next());
	}
	
	@Test
	public void testTripleSquareBracketCloseAsToken() {
		final SimpleScannerInput input = new SimpleScannerInput("] ]]");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(SquareBracketCloseToken.create(Position.create("", 1, 1, 0)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 1, 2, 1)), output.next());
		assertEquals(DoubleSquareBracketCloseToken.create(Position.create("", 1, 3, 2)), output.next());
		assertEquals(EndToken.create(Position.create("", 1, 5, 4)), output.next());
	}
	
	@Test
	public void testAsteriskAsToken() {
		final SimpleScannerInput input = new SimpleScannerInput("*");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(AsteriskToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 1 + 1, 1)), output.next());
	}
	
	@Test
	public void testMutableAsToken() {
		final SimpleScannerInput input = new SimpleScannerInput("mutable");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(MutableToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 7 + 1, 7)), output.next());
	}
	
	@Test
	public void testSymmetricAsToken() {
		final SimpleScannerInput input = new SimpleScannerInput("symmetric");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(SymmetricToken.create(Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 9 + 1, 9)), output.next());
	}
	
	@Test
	public void testWhitespaceInIdentifier() {
		final SimpleScannerInput input = new SimpleScannerInput("Hannover Hbf:class");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(IdentifierToken.create("Hannover", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create(" ", Position.create("", 0 + 1, 8 + 1, 8)), output.next());
		assertEquals(IdentifierToken.create("Hbf", Position.create("", 0 + 1, 9 + 1, 9)), output.next());
		assertEquals(ColonToken.create(Position.create("", 0 + 1, 12 + 1, 12)), output.next());
		assertEquals(ClassToken.create(Position.create("", 0 + 1, 13 + 1, 13)), output.next());
		assertEquals(EndToken.create(Position.create("", 0 + 1, 18 + 1, 18)), output.next());
	}
	
	/**
	 * Input: MyGroup:group=[MyClass:class={MyOperation:[[()->[String->Integer]!!]];};];.
	 */
	@Test
	public void testExclamationToken() {
		final SimpleScannerInput input =
				new SimpleScannerInput("MyGroup:group=[MyClass:class={MyOperation:[[()->[String->Integer]!]];};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final TokenStream stream = SimpleTokenStream.create();
		
		final IdentifierToken myGroupToken = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClassToken = IdentifierToken.create("MyClass", Position.create("", 1, 16, 15));
		final IdentifierToken myOperationToken = IdentifierToken.create("MyOperation", Position.create("", 1, 31, 30));
		
		stream.add(myGroupToken);
		stream.add(ColonToken.create(Position.create("", 1, 8, 7)));
		stream.add(GroupToken.create(Position.create("", 1, 9, 8)));
		stream.add(EqualToken.create(Position.create("", 1, 14, 13)));
		stream.add(SquareBracketOpenToken.create(Position.create("", 1, 15, 14)));
		stream.add(myClassToken);
		stream.add(ColonToken.create(Position.create("", 1, 23, 22)));
		stream.add(ClassToken.create(Position.create("", 1, 24, 23)));
		stream.add(EqualToken.create(Position.create("", 1, 29, 28)));
		stream.add(CurlyBracketOpenToken.create(Position.create("", 1, 30, 29)));
		stream.add(myOperationToken);
		stream.add(ColonToken.create(Position.create("", 1, 42, 41)));
		stream.add(DoubleSquareBracketOpenToken.create(Position.create("", 1, 43, 42)));
		stream.add(BracketOpenToken.create(Position.create("", 1, 45, 44)));
		stream.add(BracketCloseToken.create(Position.create("", 1, 46, 45)));
		stream.add(ArrowToken.create(Position.create("", 1, 47, 46)));
		stream.add(SquareBracketOpenToken.create(Position.create("", 1, 49, 48)));
		stream.add(IdentifierToken.create("String", Position.create("", 1, 50, 49)));
		stream.add(ArrowToken.create(Position.create("", 1, 56, 55)));
		stream.add(IdentifierToken.create("Integer", Position.create("", 1, 58, 57)));
		stream.add(SquareBracketCloseToken.create(Position.create("", 1, 65, 64)));
		stream.add(ExclamationToken.create(Position.create("", 1, 66, 65)));
		stream.add(DoubleSquareBracketCloseToken.create(Position.create("", 1, 67, 66)));
		stream.add(SemicolonToken.create(Position.create("", 1, 69, 68)));
		stream.add(CurlyBracketCloseToken.create(Position.create("", 1, 70, 69)));
		stream.add(SemicolonToken.create(Position.create("", 1, 71, 70)));
		stream.add(SquareBracketCloseToken.create(Position.create("", 1, 72, 71)));
		stream.add(SemicolonToken.create(Position.create("", 1, 73, 72)));
		stream.add(EndToken.create(Position.create("", 1, 74, 73)));
		
		this.assertEqualsTokenStream(stream, output);
	}
	
	/**
	 * Input: MyGroup:group=[MyClass:class={MyOperation:[[()->{TestException}!]];};TestException:exception={};];.
	 */
	@Test
	public void testExclamationToken2() {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"MyGroup:group=[MyClass:class={MyOperation:[[()->{TestException}!]];};TestException:exception={};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final TokenStream stream = SimpleTokenStream.create();
		
		final IdentifierToken myGroupToken = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClassToken = IdentifierToken.create("MyClass", Position.create("", 1, 16, 15));
		final IdentifierToken myOperationToken = IdentifierToken.create("MyOperation", Position.create("", 1, 31, 30));
		final IdentifierToken testExceptionReferenceToken =
				IdentifierToken.create("TestException", Position.create("", 1, 50, 49));
		final IdentifierToken testExceptionToken =
				IdentifierToken.create("TestException", Position.create("", 1, 70, 69));
		final CurlyBracketOpenToken sumTypeStart = CurlyBracketOpenToken.create(Position.create("", 1, 49, 48));
		
		stream.add(myGroupToken);
		stream.add(ColonToken.create(Position.create("", 1, 8, 7)));
		stream.add(GroupToken.create(Position.create("", 1, 9, 8)));
		stream.add(EqualToken.create(Position.create("", 1, 14, 13)));
		stream.add(SquareBracketOpenToken.create(Position.create("", 1, 15, 14)));
		stream.add(myClassToken);
		stream.add(ColonToken.create(Position.create("", 1, 23, 22)));
		stream.add(ClassToken.create(Position.create("", 1, 24, 23)));
		stream.add(EqualToken.create(Position.create("", 1, 29, 28)));
		stream.add(CurlyBracketOpenToken.create(Position.create("", 1, 30, 29)));
		stream.add(myOperationToken);
		stream.add(ColonToken.create(Position.create("", 1, 42, 41)));
		stream.add(DoubleSquareBracketOpenToken.create(Position.create("", 1, 43, 42)));
		stream.add(BracketOpenToken.create(Position.create("", 1, 45, 44)));
		stream.add(BracketCloseToken.create(Position.create("", 1, 46, 45)));
		stream.add(ArrowToken.create(Position.create("", 1, 47, 46)));
		stream.add(sumTypeStart);
		stream.add(testExceptionReferenceToken);
		stream.add(CurlyBracketCloseToken.create(Position.create("", 1, 63, 62)));
		stream.add(ExclamationToken.create(Position.create("", 1, 64, 63)));
		stream.add(DoubleSquareBracketCloseToken.create(Position.create("", 1, 65, 64)));
		stream.add(SemicolonToken.create(Position.create("", 1, 67, 66)));
		stream.add(CurlyBracketCloseToken.create(Position.create("", 1, 68, 67)));
		stream.add(SemicolonToken.create(Position.create("", 1, 69, 68)));
		stream.add(testExceptionToken);
		stream.add(ColonToken.create(Position.create("", 1, 83, 82)));
		stream.add(ExceptionToken.create(Position.create("", 1, 84, 83)));
		stream.add(EqualToken.create(Position.create("", 1, 93, 92)));
		stream.add(CurlyBracketOpenToken.create(Position.create("", 1, 94, 93)));
		stream.add(CurlyBracketCloseToken.create(Position.create("", 1, 95, 94)));
		stream.add(SemicolonToken.create(Position.create("", 1, 96, 95)));
		stream.add(SquareBracketCloseToken.create(Position.create("", 1, 97, 96)));
		stream.add(SemicolonToken.create(Position.create("", 1, 98, 97)));
		stream.add(EndToken.create(Position.create("", 1, 99, 98)));
		
		this.assertEqualsTokenStream(stream, output);
	}
	
	/**
	 * Input: MyGroup:group!!!!{};];.
	 */
	@Test
	public void testExclamationToken3() {
		final SimpleScannerInput input = new SimpleScannerInput("MyGroup:group!!!!{};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final TokenStream stream = SimpleTokenStream.create();
		
		final IdentifierToken myGroupToken = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		
		stream.add(myGroupToken);
		stream.add(ColonToken.create(Position.create("", 1, 8, 7)));
		stream.add(GroupToken.create(Position.create("", 1, 9, 8)));
		stream.add(ExclamationToken.create(Position.create("", 1, 14, 13)));
		stream.add(ExclamationToken.create(Position.create("", 1, 15, 14)));
		stream.add(ExclamationToken.create(Position.create("", 1, 16, 15)));
		stream.add(ExclamationToken.create(Position.create("", 1, 17, 16)));
		stream.add(CurlyBracketOpenToken.create(Position.create("", 1, 18, 17)));
		stream.add(CurlyBracketCloseToken.create(Position.create("", 1, 19, 18)));
		stream.add(SemicolonToken.create(Position.create("", 1, 20, 19)));
		stream.add(SquareBracketCloseToken.create(Position.create("", 1, 21, 20)));
		stream.add(SemicolonToken.create(Position.create("", 1, 22, 21)));
		stream.add(EndToken.create(Position.create("", 1, 23, 22)));
		
		this.assertEqualsTokenStream(stream, output);
	}
	
	/**
	 * Check if two TokenStream contains the same tokens in the same order.
	 * 
	 * @param expected
	 *            the expected tokenstream
	 * @param actual
	 *            the actual tokenstream
	 */
	private void assertEqualsTokenStream(final TokenStream expected, final TokenStream actual) {
		
		while (expected.hasNext()) {
			assertTrue(actual.hasNext());
			assertEquals(expected.next(), actual.next());
		}
		
		assertTrue(!actual.hasNext());
	}
}
