package de.fhdw.wtf.walker.tasks.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException;
import de.fhdw.wtf.common.exception.walker.DoubleAttributenameException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.GroupDependencyTask;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.BaseTypeInheritanceCheck;
import de.fhdw.wtf.walker.tasks.CyclicInheritanceCheck;
import de.fhdw.wtf.walker.tasks.DoubleAttributenameCheck;
import de.fhdw.wtf.walker.tasks.DoubleGroupcomponentCheck;
import de.fhdw.wtf.walker.tasks.TypeReferencer;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;

/**
 * Test {@link DoubleAttributenameCheck}.
 */
public class TestDoubleAttributenameCheck extends TestCase {
	
	/**
	 * Frist 2 tests check when a name contains another name it works.
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 **/
	@Test
	public void testDoubleAttributename1() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "class1:class={" + "attribute: Integer;"
						+ "attributeLong: Integer;" + "};" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		baseTypeInheritanceCheck.addDependency(referencer);
		cyclicInheritance.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * "" + "group1:group=[" + "class1:class={" + "attributelong: Integer;" + "attribute: Integer;" + "};" + "];" .
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 */
	@Test
	public void testDoubleAttributename2() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "class1:class={" + "attributelong: Integer;"
						+ "attribute: Integer;" + "};" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		baseTypeInheritanceCheck.addDependency(referencer);
		cyclicInheritance.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Checks wether you can only use a specific attributename one time per class.
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 */
	@Test
	public void testDoubleAttributename3() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "class1:class={" + "attribute: Integer;"
						+ "attribute: Integer;" + "};" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		cyclicInheritance.addDependency(referencer);
		baseTypeInheritanceCheck.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(5, okResult.size());
		assertEquals(1, failResult.size());
		
		final ExceptionalTaskResult exceptionalTaskResult = failResult.iterator().next();
		if (!(exceptionalTaskResult.getError() instanceof DoubleAttributenameException)) {
			fail();
		}
	}
	
	/**
	 * Checks wether you can use a specific attribute name in each class.
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 */
	@Test
	public void testDoubleAttributename4() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "class1:class={" + "attribute: Integer;" + "};"
						+ "class2:class={" + "attribute: Integer;" + "};" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		cyclicInheritance.addDependency(referencer);
		baseTypeInheritanceCheck.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * group1:group=[class1:class=class2+{attribute: Integer;};class2:class={attribute: Integer;};]; .
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 */
	@Test
	public void testDoubleAttributenameInInheritance1() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class=class2+{attribute: Integer;};"
						+ "class2:class={attribute: Integer;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		baseTypeInheritanceCheck.addDependency(referencer);
		cyclicInheritance.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
		
	}
	
	/**
	 * group1:group=[class1:class=class2+class3{attribute: Integer;}; class2:class={attribute2: Integer;};
	 * class3:class={attribute: Integer;};]; .
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 */
	@Test
	public void testDoubleAttributenameInInheritance2() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class=class2+class3+{attribute: Integer;};"
						+ "class2:class={attribute2: Integer;}; class3:class={attribute: Integer;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		cyclicInheritance.addDependency(referencer);
		baseTypeInheritanceCheck.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
		
	}
	
	/**
	 * group1:group=[class1:class=class2+{attribute: Integer;};class2:class={attribute2: Integer;};]; .
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 * 
	 */
	@Test
	public void testDoubleAttributenameInInheritance3() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class=class2+{attribute: Integer;};"
						+ "class2:class={attribute2: Integer;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		cyclicInheritance.addDependency(referencer);
		baseTypeInheritanceCheck.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * group1:group=[class1:class=class2+{attribute: Integer;}; class2:class=class3+{attribute2: Integer;};
	 * class3:class={attribute: Integer;};]; .
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 */
	@Test
	public void testDoubleAttributenameInInheritance4() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class=class2+{attribute: Integer;};"
						+ "class2:class=class3+{attribute2: Integer;}; class3:class={attribute: Integer;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		cyclicInheritance.addDependency(referencer);
		baseTypeInheritanceCheck.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
		
	}
	
	/**
	 * group1:group=[class1:class=class2+class3{attribute: Integer;}; class2:class={attribute2: Integer;};
	 * class3:class={attribute2: Integer;};]; .
	 * 
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws CyclicPartDefinitionException
	 *             CyclicPartDefinitionException
	 */
	@Test
	public void testDoubleAttributenameInInheritance5() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class=class2+class3+{attribute: Integer;};"
						+ "class2:class={attribute2: Integer;}; class3:class={attribute2: Integer;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final DoubleAttributenameCheck doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final GroupDependencyTask inheritanceChecker = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		
		referencer.addDependency(doubleGroupcomponent);
		cyclicInheritance.addDependency(referencer);
		baseTypeInheritanceCheck.addDependency(referencer);
		inheritanceChecker.addMembers(cyclicInheritance, baseTypeInheritanceCheck);
		
		doubleAttributenameCheck.addDependency(inheritanceChecker);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(0, failResult.size());
		
	}
}
